local locale = GetLocale();
if (locale == "deDE") then
    L = MountJournalEnhancedLocals["deDE"];
else
    L = MountJournalEnhancedLocals["enUS"];
end

local private = {
    initiated = false,
    hooks = {},
    hiddenMounts = {},
    filter = {
        onlyFavorites = false,
        source = {},
        additionalSource = {
            pvp = true,
        },
        onlyUsable = false,
        faction = {
            alliance = true,
            horde = true,
            noFaction = true,
        },
        mountType = {
            ground = true,
            flying = true,
            waterWalking = true,
            underwater = true,
            transform = true,
        },
        family = {},
        hidden = false,
    },
    characterMountCountFrame = nil,
    mountAchievementStatusFrame = nil,
};

function private:MountJournal_GetMountInfoHook(index)
    local creatureName, spellID, icon, active, isUsable, sourceType, isFavorite, isFactionSpecific, faction, hideOnChar, isCollected = self.hooks["MountJournal_GetMountInfo"](index);
    isUsable = isUsable and IsUsableSpell(spellID);
    return creatureName, spellID, icon, active, isUsable, sourceType, isFavorite, isFactionSpecific, faction, hideOnChar, isCollected;
end

function private:MountJournalFilterDropDown_InitializeHook(this, level)
    local info = UIDropDownMenu_CreateInfo();
    info.keepShownOnClick = true;

    if level == 1 then
        info.text = COLLECTED
        info.func = function(_, _, _, value)
            MountJournal_SetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_COLLECTED,value);
            MountJournal_UpdateMountList();

            if (value) then
                UIDropDownMenu_EnableButton(1,2);
            else
                UIDropDownMenu_DisableButton(1,2);
            end;
        end 
        info.checked = MountJournal_GetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_COLLECTED);
        info.isNotRadio = true;
        UIDropDownMenu_AddButton(info, level)
            
        info.text = FAVORITES_FILTER;
        info.leftPadding = 16;
        info.disabled = not C_MountJournal.GetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_COLLECTED);
        info.checked = self.filter.onlyFavorites;
        info.isNotRadio = true;
        info.notCheckable = false;
        info.hasArrow = false;
        info.func = function(_, _, _, value)
            self.filter.onlyFavorites = value;
            MountJournal_DirtyList(MountJournal);
            MountJournal_UpdateMountList();
         end 
        UIDropDownMenu_AddButton(info, level);
        
        info.leftPadding = 0;
        info.disabled = false;
        
        info.text = NOT_COLLECTED
        info.func = function(_, _, _, value)
             MountJournal_SetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_NOT_COLLECTED,value);
             MountJournal_UpdateMountList();
         end 
        info.checked = MountJournal_GetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_NOT_COLLECTED);
        info.isNotRadio = true;
        UIDropDownMenu_AddButton(info, level)
        
        info.text = L["Only usable"];
        info.func = function(_, _, _, value)
            self.filter.onlyUsable = value;
            MountJournal_DirtyList(MountJournal);
            MountJournal_UpdateMountList();
        end 
        info.checked = self.filter.onlyUsable;
        info.isNotRadio = true;
        info.notCheckable = false;
        info.hasArrow = false;
        UIDropDownMenu_AddButton(info, level);
        
        info.text = SOURCES;
        info.checked =     nil;
        info.isNotRadio = nil;
        info.func =  nil;
        info.hasArrow = true;
        info.notCheckable = true;
        info.value = 1;
        UIDropDownMenu_AddButton(info, level)
    
        info.text = TYPE;
        info.isNotRadio = true;
        info.notCheckable = true;
        info.hasArrow = true;
        info.value = 2;
        UIDropDownMenu_AddButton(info, level);
    
        info.text = FACTION;
        info.isNotRadio = true;
        info.notCheckable = true;
        info.hasArrow = true;
        info.value = 3;
        UIDropDownMenu_AddButton(info, level);

        info.text = L["Family"];
        info.isNotRadio = true;
        info.notCheckable = true;
        info.hasArrow = true;
        info.value = 4;
        UIDropDownMenu_AddButton(info, level);
        
        info.text = L["Hidden"];
        info.checked =     self.filter.hidden;
        info.isNotRadio = true;
        info.notCheckable = false;
        info.hasArrow = false;
        info.func = function(_, _, _, value)
            self.filter.hidden = value;
            MountJournal_DirtyList(MountJournal);
            MountJournal_UpdateMountList();
         end 
        UIDropDownMenu_AddButton(info, level);
    else
        if (UIDROPDOWNMENU_MENU_VALUE == 1) then
            info.hasArrow = false;
            info.isNotRadio = true;
            info.notCheckable = true;
                
            info.text = CHECK_ALL
            info.func = function()
                for k, v in pairs(self.filter.additionalSource) do
                    self.filter.additionalSource[k] = true;
                end
            
                MountJournal_AddAllSources();
                UIDropDownMenu_Refresh(MountJournalFilterDropDown, 1, 2);
                MountJournal_UpdateMountList();
            end
            UIDropDownMenu_AddButton(info, level)
            
            info.text = UNCHECK_ALL
            info.func = function()
                for k, v in pairs(self.filter.additionalSource) do
                    self.filter.additionalSource[k] = false;
                end
            
                MountJournal_ClearAllSources();
                UIDropDownMenu_Refresh(MountJournalFilterDropDown, 1, 2);
                MountJournal_UpdateMountList();
            end
            UIDropDownMenu_AddButton(info, level)
            
            info.text = PVP;
            info.func = function(_, _, _, value)
                self.filter.additionalSource.pvp = value;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end 
            info.checked = function() return self.filter.additionalSource.pvp; end;
            info.isNotRadio = true;
            info.notCheckable = false;
            info.hasArrow = false;
            UIDropDownMenu_AddButton(info, level);
            
            info.notCheckable = false;
            MountJournal_CollectAvailableFilters();
            local numSources = C_PetJournal.GetNumPetSources();
            for i=1,numSources do
                if ( MountJournal.baseFilterTypes[i] ) then
                    info.text = _G["BATTLE_PET_SOURCE_"..i];
                    info.func = function(_, _, _, value)
                                    MountJournal_SetSourceFilter(i,value);
                                    MountJournal_UpdateMountList();
                                end
                    info.checked = function() return MountJournal_IsSourceNotFiltered(i) end;
                    UIDropDownMenu_AddButton(info, level);
                end
            end
        end
        
        if (UIDROPDOWNMENU_MENU_VALUE == 2) then
            info.hasArrow = false;
            info.isNotRadio = true;
            info.notCheckable = true;
                
            info.text = CHECK_ALL
            info.func = function()
                for k, v in pairs(self.filter.mountType) do
                    self.filter.mountType[k] = true;
                end
                UIDropDownMenu_Refresh(MountJournalFilterDropDown, 2, 2);
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end
            UIDropDownMenu_AddButton(info, level)
            
            info.text = UNCHECK_ALL
            info.func = function()
                for k, v in pairs(self.filter.mountType) do
                    self.filter.mountType[k] = false;
                end
                UIDropDownMenu_Refresh(MountJournalFilterDropDown, 2, 2);
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end
            UIDropDownMenu_AddButton(info, level)
        
            info.text = L["Ground"];
            info.func = function(_, _, _, value)
                self.filter.mountType.ground = value;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end 
            info.checked = function() return self.filter.mountType.ground; end;
            info.isNotRadio = true;
            info.notCheckable = false;
            info.hasArrow = false;
            UIDropDownMenu_AddButton(info, level);
            
            info.text = L["Flying"];
            info.func = function(_, _, _, value)
                self.filter.mountType.flying = value;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end 
            info.checked = function() return self.filter.mountType.flying; end;
            info.isNotRadio = true;
            info.notCheckable = false;
            info.hasArrow = false;
            UIDropDownMenu_AddButton(info, level);
            
            info.text = L["Water Walking"];
            info.func = function(_, _, _, value)
                self.filter.mountType.waterWalking = value;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end 
            info.checked = function() return self.filter.mountType.waterWalking; end;
            info.isNotRadio = true;
            info.notCheckable = false;
            info.hasArrow = false;
            UIDropDownMenu_AddButton(info, level);
            
            info.text = L["Underwater"];
            info.func = function(_, _, _, value)
                self.filter.mountType.underwater = value;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end 
            info.checked = function() return self.filter.mountType.underwater; end;
            info.isNotRadio = true;
            info.notCheckable = false;
            info.hasArrow = false;
            UIDropDownMenu_AddButton(info, level);
            
            info.text = L["Transform"];
            info.func = function(_, _, _, value)
                self.filter.mountType.transform = value;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end 
            info.checked = function() return self.filter.mountType.transform; end;
            info.isNotRadio = true;
            info.notCheckable = false;
            info.hasArrow = false;
            UIDropDownMenu_AddButton(info, level);            
        end
        
        if (UIDROPDOWNMENU_MENU_VALUE == 3) then
            info.text = FACTION_ALLIANCE;
            info.func = function(_, _, _, value)
                self.filter.faction.alliance = value;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end 
            info.checked = self.filter.faction.alliance;
            info.isNotRadio = true;
            info.notCheckable = false;
            info.hasArrow = false;
            UIDropDownMenu_AddButton(info, level);
            
            info.text = FACTION_HORDE;
            info.func = function(_, _, _, value)
                self.filter.faction.horde = value;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end 
            info.checked = self.filter.faction.horde;
            info.isNotRadio = true;
            info.notCheckable = false;
            info.hasArrow = false;
            UIDropDownMenu_AddButton(info, level);
            
            info.text = NPC_NAMES_DROPDOWN_NONE;
            info.func = function(_, _, _, value)
                self.filter.faction.noFaction = value;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end 
            info.checked = self.filter.faction.noFaction;
            info.isNotRadio = true;
            info.notCheckable = false;
            info.hasArrow = false;
            UIDropDownMenu_AddButton(info, level);
        end
        
        if (UIDROPDOWNMENU_MENU_VALUE == 4) then
            info.hasArrow = false;
            info.isNotRadio = true;
            info.notCheckable = true;
                
            info.text = CHECK_ALL
            info.func = function()
                for k, v in pairs(self.filter.family) do
                    self.filter.family[k] = true;
                end
                UIDropDownMenu_Refresh(MountJournalFilterDropDown, 4, 2);
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end
            UIDropDownMenu_AddButton(info, level)
            
            info.text = UNCHECK_ALL
            info.func = function()
                for k, v in pairs(self.filter.family) do
                    self.filter.family[k] = false;
                end
                UIDropDownMenu_Refresh(MountJournalFilterDropDown, 4, 2);
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end
            UIDropDownMenu_AddButton(info, level)
        
            local sortedFamilies = {};
            for family, _ in pairs(MountJournalEnhancedFamilyData) do
                table.insert(sortedFamilies, family);
            end
            table.sort(sortedFamilies, function(a, b) return (L[a] or a) < (L[b] or b); end);
            
            for _, family in pairs(sortedFamilies) do
                info.text = L[family] or family;
                info.func = function(_, _, _, value)
                    self.filter.family[family] = value;
                    MountJournal_DirtyList(MountJournal);
                    MountJournal_UpdateMountList();
                end 
                info.checked = function() return self.filter.family[family] end;
                info.isNotRadio = true;
                info.notCheckable = false;
                info.hasArrow = false;
                UIDropDownMenu_AddButton(info, level);
            end
            
            self:MakeMultiColumnMenu(level, 20);
        end
    end
end

function private:MakeMultiColumnMenu(level, entriesPerColumn)
    local listFrame = _G["DropDownList"..level];
    local columnWidth = listFrame.maxWidth + 25;
        
    local listFrameName = listFrame:GetName();
    local columnIndex = 0;
    for index = entriesPerColumn + 1, listFrame.numButtons do
        columnIndex = math.ceil(index / entriesPerColumn);
        local button = _G[listFrameName.."Button"..index];
        local yPos = -((button:GetID() - 1 - entriesPerColumn * (columnIndex - 1)) * UIDROPDOWNMENU_BUTTON_HEIGHT) - UIDROPDOWNMENU_BORDER_HEIGHT;
        
        button:ClearAllPoints();
        button:SetPoint("TOPLEFT", button:GetParent(), "TOPLEFT", columnWidth * (columnIndex - 1), yPos);
        button:SetWidth(columnWidth);
    end
    
    listFrame:SetHeight((min(listFrame.numButtons, entriesPerColumn) * UIDROPDOWNMENU_BUTTON_HEIGHT) + (UIDROPDOWNMENU_BORDER_HEIGHT * 2));
    listFrame:SetWidth(columnWidth * columnIndex);
    
    self:hook(nil, "UIDropDownMenu_OnHide", function(this)
        self:unhook(listFrame, "SetWidth");
        self:unhook(nil, "UIDropDownMenu_OnHide");
        UIDropDownMenu_OnHide(this);
    end);
    self:hook(listFrame, "SetWidth", function() end);
end

function private:MountOptionsMenu_InitHook(this, level)
    local info = UIDropDownMenu_CreateInfo();
    info.notCheckable = true;
        
    info.text = BATTLE_PET_SUMMON;
    info.func = function() MountJournal_Summon(MountJournal.menuMountID) end
    if (MountJournal.menuMountID and MountJournal.active) then
        info.text = PET_DISMISS;
        info.func = function() MountJournal_Dismiss() end
    end
    if ( MountJournal.menuMountID and MountJournal.menuIsUsable ) then
        info.disabled = false;
    else
        info.disabled = true;
    end
    UIDropDownMenu_AddButton(info, level);
    
    local canFavorite = false;
    local isFavorite = false;
    if (MountJournal.menuMountID) then
         isFavorite, canFavorite = MountJournal_GetIsFavorite(MountJournal.menuMountID);
    end

    local spellId = nil;
    local isCollected = false;
    if (MountJournal.menuMountID) then
        _, spellId, _, _, _, _, _, _, _, _, isCollected = MountJournal_GetMountInfo(MountJournal.menuMountID);
    end
    
    if (isFavorite) then
        info.text = BATTLE_PET_UNFAVORITE;
        info.func = function() 
            MountJournal_SetIsFavorite(MountJournal.menuMountID, false);
        end
    else
        info.text = BATTLE_PET_FAVORITE;
        info.func = function() 
            MountJournal_SetIsFavorite(MountJournal.menuMountID, true);
        end
    end

    info.disabled = not isCollected or not canFavorite;
    
    UIDropDownMenu_AddButton(info, level);
    
    if (spellId) then   
        info.disabled = nil;
        if (self.hiddenMounts[spellId]) then
            info.text = L["Show"];
            info.func = function() 
                self.hiddenMounts[spellId] = false;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end;
        else
            info.text = L["Hide"];
            info.func = function() 
                self.hiddenMounts[spellId] = true;
                MountJournal_DirtyList(MountJournal);
                MountJournal_UpdateMountList();
            end;
        end
        UIDropDownMenu_AddButton(info, level);
    end
    
    info.disabled = nil;
    info.text = CANCEL
    info.func = nil
    UIDropDownMenu_AddButton(info, level)
end

function private:MountJournal_UpdateMountListHook()
    self.hooks["MountJournal_UpdateMountList"]();
    
    local buttons = MountJournal.ListScrollFrame.buttons;
    for i = 1, #buttons do
        if (self.hiddenMounts[buttons[i].spellID]) then
            buttons[i].DragButton.IsHidden:SetShown(true);
        else
            buttons[i].DragButton.IsHidden:SetShown(false);
        end
    end
end

function private:filterBySource(spellId)
    for source, value in pairs(self.filter.additionalSource) do   
        if (MountJournalEnhancedSourceData[source]) then
            if (MountJournalEnhancedSourceData[source][spellId]) then
                if (value) then
                    return true;
                end
            end
        end
    end
    
    return false;
end

function private:filterByFamily(spellId)
    local notContainded = true;

    local allDisabled = true;
    local allEnabled = true;
    for family, value in pairs(self.filter.family) do
        if (value) then
            allDisabled = false;
        else
            allEnabled = false;
        end
    
        if (MountJournalEnhancedFamilyData[family]) then
            if (MountJournalEnhancedFamilyData[family][spellId]) then
                if (value) then
                    return true;
                else
                    notContainded = false;
                end
            end
        end
    end
    
    return notContainded and (allDisabled or allEnabled);
end

function private:MountJournal_UpdateCachedListHook(this)
    if ( this.cachedMounts and not this.dirtyList ) then
        return;
    end
    this.cachedMounts = {};
    this.sortVal = {};
    this.numOwned = 0;

    local showCollected = C_MountJournal.GetCollectedFilterSetting(LE_MOUNT_JOURNAL_FILTER_COLLECTED);
    for i = 1, MountJournal_GetNumMounts() do
        local creatureName, spellId, icon, active, isUsable, sourceType, isFavorite, isFaction, faction, hideOnChar, isCollected = MountJournal_GetMountInfo(i);
        local _, _, _, isSelfMount, mountType = C_MountJournal.GetMountInfoExtra(i);
        
        isUsable = isUsable and IsUsableSpell(spellId);
        
        if ( hideOnChar ~= true and (MountJournal_MountMatchesFilter(this, creatureName, sourceType, isCollected) or self:filterBySource(spellId)) and 
            (self.filter.faction.noFaction and not isFaction or self.filter.faction.alliance and faction == 1 or self.filter.faction.horde and faction == 0) and
            (not self.filter.onlyFavorites or isFavorite or not showCollected) and
            (not self.filter.onlyUsable or isUsable) and
            (self.filter.hidden or not self.hiddenMounts[spellId]) and
            ((self.filter.mountType.ground and (mountType == 230 or mountType == 231 or mountType == 241 or mountType == 269)) or 
                (self.filter.mountType.flying and (mountType == 247 or mountType == 248)) or 
                (self.filter.mountType.waterWalking and mountType == 269) or 
                -- 64731 Sea Turtle, 30174 Riding Turtle
                (self.filter.mountType.underwater and (mountType == 232 or mountType == 254 or spellId == 64731 or spellId == 30174)) or
                (self.filter.mountType.transform and isSelfMount) or
                (mountType ~= 230 and mountType ~= 231 and mountType ~= 241 and mountType ~= 269 and mountType ~= 247 and mountType ~= 248 and mountType ~= 232 and mountType ~= 254)) and
            self:filterByFamily(spellId))
        then
            this.cachedMounts[#this.cachedMounts + 1] = i;
            this.sortVal[i] = MountJournal_GetMountSortVal(this, isUsable, sourceType, isFavorite, isCollected);
        end
        if (isCollected and hideOnChar ~= true) then
            this.numOwned = this.numOwned + 1;
        end
    end

    local comparison = function(index1, index2)
        return MountJournal_SortComparison(this, index1, index2);
    end

    table.sort(this.cachedMounts, comparison);
    this.sortVal = {};
    
    this.dirtyList = false;
end

function private:MountListDragButton_OnClickHook(this, button)   
    local parent = this:GetParent();
    if ( button ~= "LeftButton" ) then
        MountJournal_ShowMountDropdown(parent.index, this, 0, 0);
    elseif ( IsModifiedClick("CHATLINK") ) then
        local id = parent.spellID;
        if ( MacroFrame and MacroFrame:IsShown() ) then
            local spellName = GetSpellInfo(id);
            ChatEdit_InsertLink(spellName);
        else
            local spellLink = GetSpellLink(id)
            ChatEdit_InsertLink(spellLink);
        end
    else
        MountJournal_Pickup(parent.index);
    end
end

function private:MountListItem_OnClickHook(this, button)   
    if ( button ~= "LeftButton" ) then
        MountJournal_ShowMountDropdown(this.index, this, 0, 0);
    elseif ( IsModifiedClick("CHATLINK") ) then
        local id = this.spellID;
        if ( MacroFrame and MacroFrame:IsShown() ) then
            local spellName = GetSpellInfo(id);
            ChatEdit_InsertLink(spellName);
        else
            local spellLink = GetSpellLink(id)
            ChatEdit_InsertLink(spellLink);
        end
    elseif ( this.spellID ~= MountJournal_GetSelectedSpellID() ) then
        MountJournal_Select(this.index);
    end
end

function private:init()
    if (self.filter.source and #self.filter.source > 0) then
        MountJournal.filterTypes = self.filter.source;
    end
    
    self:hook(nil, "MountJournal_GetMountInfo", function(index) return self:MountJournal_GetMountInfoHook(index); end);
    
    self:hook(nil, "MountJournal_UpdateMountList", function() self:MountJournal_UpdateMountListHook(); end);
    MountJournal.ListScrollFrame.update = function() self:MountJournal_UpdateMountListHook() end;
    
    self:hook(nil, "MountJournal_UpdateCachedList", function(this) self:MountJournal_UpdateCachedListHook(this); end);
    
    self:hook(nil, "MountJournalFilterDropDown_Initialize", function(this, level) self:MountJournalFilterDropDown_InitializeHook(this, level); end);
    MountJournalFilterDropDown_OnLoad(MountJournalFilterDropDown);
    
    self:hook(nil, "MountOptionsMenu_Init", function(this, level) self:MountOptionsMenu_InitHook(this, level); end);
    UIDropDownMenu_Initialize(MountJournal.mountOptionsMenu, MountOptionsMenu_Init, "MENU");
    
    self:hook(nil, "MountListItem_OnClick", function(this, button) self:MountListItem_OnClickHook(this, button); end);
    self:hook(nil, "MountListDragButton_OnClick", function(this, button) self:MountListDragButton_OnClickHook(this, button); end);
    
    local buttons = MountJournal.ListScrollFrame.buttons;
    for i = 1, #buttons do
        buttons[i]:SetScript("OnClick", function(this, button) MountListItem_OnClick(this, button); end);
        buttons[i]:SetScript("OnDoubleClick", function(this, button) if (button == "LeftButton") then MountJournal_Summon(this.index); end end);
        buttons[i].DragButton:SetScript("OnClick", function(this, button) MountListDragButton_OnClick(this, button); end);
        buttons[i].DragButton.IsHidden = buttons[i].DragButton:CreateTexture(nil, "OVERLAY");
        buttons[i].DragButton.IsHidden:SetTexture("Interface\\BUTTONS\\UI-GroupLoot-Pass-Up");
        buttons[i].DragButton.IsHidden:SetSize(36, 36);
        buttons[i].DragButton.IsHidden:SetPoint("CENTER", buttons[i].DragButton, "CENTER", 0, 0);
        buttons[i].DragButton.IsHidden:SetDrawLayer("OVERLAY", 1);
        buttons[i].DragButton.IsHidden:SetShown(false);
    end
    
    MountJournal_DirtyList(MountJournal);
    MountJournal_UpdateMountList();
    
    self:initCharacterMountCount();
    self:initMountAchievementStatus();
end

function private:initCharacterMountCount()
    self.characterMountCountFrame = CreateFrame("frame", "MJECharacterMountCount", MountJournal, "InsetFrameTemplate3");
    
    local frame = self.characterMountCountFrame;
    frame:ClearAllPoints();
    frame:SetPoint("TOPLEFT", MountJournal, 70, -42);
    frame:SetSize(130, 18);
    
    frame.staticText = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall");
    frame.staticText:ClearAllPoints();
    frame.staticText:SetPoint("LEFT", frame, 10, 0);
    frame.staticText:SetText(CHARACTER);
    
    frame.uniqueCount = frame:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall");
    frame.uniqueCount:ClearAllPoints();
    frame.uniqueCount:SetPoint("RIGHT", frame, -10, 0);
    frame.uniqueCount:SetText(GetNumCompanions("MOUNT"));
    
    MountJournal.MountCount:SetPoint("TopLeft", 70, -22);
    MountJournal.MountCount:SetSize(130, 18);
end

function private:initMountAchievementStatus()
    self.mountAchievementStatusFrame = CreateFrame("Button", "MJEMountAchievementStatus", MountJournal);
    
    local frame = self.mountAchievementStatusFrame;
    frame:ClearAllPoints();
    frame:SetPoint("TOP", MountJournal, 0, -21);
    frame:SetSize(60, 40);
    
    frame.bgLeft = frame:CreateTexture(nil, "BACKGROUND");
    frame.bgLeft:SetAtlas("PetJournal-PetBattleAchievementBG");
    frame.bgLeft:ClearAllPoints();
    frame.bgLeft:SetSize(46, 18);
    frame.bgLeft:SetPoint("Top", -56, -12);
    frame.bgLeft:SetVertexColor(1, 1, 1, 1);
    
    frame.bgRight = frame:CreateTexture(nil, "BACKGROUND");
    frame.bgRight:SetAtlas("PetJournal-PetBattleAchievementBG");
    frame.bgRight:ClearAllPoints();
    frame.bgRight:SetSize(46, 18);
    frame.bgRight:SetPoint("Top", 55, -12);
    frame.bgRight:SetVertexColor(1, 1, 1, 1);
    frame.bgRight:SetTexCoord(1, 0, 0, 1);
    
    frame.highlight = frame:CreateTexture(nil);
    frame.highlight:SetDrawLayer("BACKGROUND", 1);
    frame.highlight:SetAtlas("PetJournal-PetBattleAchievementGlow");
    frame.highlight:ClearAllPoints();
    frame.highlight:SetSize(210, 40);
    frame.highlight:SetPoint("CENTER", 0, 0);
    frame.highlight:SetShown(false);
    
    frame.icon = frame:CreateTexture(nil, "OVERLAY");
    frame.icon:SetTexture("Interface\\AchievementFrame\\UI-Achievement-Shields-NoPoints");
    frame.icon:ClearAllPoints();
    frame.icon:SetSize(30, 30);
    frame.icon:SetPoint("RIGHT", 0, -5);
    frame.icon:SetTexCoord(0, 0.5, 0, 0.5);
    
    frame.staticText = frame:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");
    frame.staticText:ClearAllPoints();
    frame.staticText:SetPoint("RIGHT", frame.icon, "LEFT", -4, 4);
    frame.staticText:SetSize(0, 0);
    frame.staticText:SetText(self:getAchievementPoints());
    
    frame:SetScript("OnClick", function()
        ToggleAchievementFrame();
        local i = 1;
        local button = _G["AchievementFrameCategoriesContainerButton" .. i];
        while button do
            if button.element.id == 15246 then -- Collections
                button:Click();
                button = nil;
            else
                i = i + 1;
                button = _G["AchievementFrameCategoriesContainerButton" .. i];
            end
        end
        
        i = 1;
        button = _G["AchievementFrameCategoriesContainerButton" .. i];
        while button do
            if button.element.id == 15248 then -- Mounts
                button:Click();
                return;
            end
            
            i = i + 1;
            button = _G["AchievementFrameCategoriesContainerButton" .. i];
        end
    end);
    frame:SetScript("OnEnter", function() frame.highlight:SetShown(true); end);
    frame:SetScript("OnLeave", function() frame.highlight:SetShown(false); end);
end

function private:getAchievementPoints()
    local achievementsPoints = 0;
    for i = 1, (GetCategoryNumAchievements(15248)) do
        local achievementId = GetAchievementInfo(15248, i);
        achievementsPoints = self:getAchievementPointsRec(achievementId, achievementsPoints);
    end
    
    return achievementsPoints;
end

function private:getAchievementPointsRec(achievementId, achievementsPoints)
    if (not achievementId) then
        return 0;
    end
    
    local _, _, points, completed = GetAchievementInfo(achievementId);
    if (completed) then
        achievementsPoints = achievementsPoints + points;
        local previousAchievementId = GetPreviousAchievement(achievementId);
        if (previousAchievementId) then
            return self:getAchievementPointsRec(previousAchievementId, achievementsPoints);
        end
    end
    
    return achievementsPoints;
end

function private:hook(obj, name, func)
    local hook = self.hooks[name];
    if (hook ~= nil) then
        return false;
    end

    if (obj == nil) then
        self.hooks[name] = _G[name];
        _G[name] = func;
    else
        self.hooks[name] = obj[name];
        obj[name] = func;
    end
    
    return true;
end

function private:unhook(obj, name)
    local hook = self.hooks[name];
    if (hook == nil) then
        return false;
    end
    
    if (obj == nil) then
        _G[name] = hook;
    else
        obj[name] = hook;
    end
    self.hooks[name] = nil;
    
    return true;
end

function private:Load()
    if (MountJournalEnhancedSettings.hiddenMounts ~= nil) then
        self.hiddenMounts = MountJournalEnhancedSettings.hiddenMounts;
    end
    
    if (MountJournalEnhancedSettings.filter ~= nil) then
        self.filter = MountJournalEnhancedSettings.filter;
    end
    
    if (not self.filter.additionalSource) then
        self.filter.additionalSource = {
            pvp = true,
        };
    end
    
    if (not self.filter.mountType) then
        self.filter.mountType = {
            ground = true,
            flying = true,
            waterWalking = true,
            underwater = true,
            transform = true,
        };
    end
    
    if (not self.filter.family) then
        self.filter.family = {};
    end
    for family, mounts in pairs(MountJournalEnhancedFamilyData) do
        if (self.filter.family[family] == nil) then
            self.filter.family[family] = true;
        end
    end
end

function private:Unload()
    MountJournalEnhancedSettings.hiddenMounts = self.hiddenMounts;
    MountJournalEnhancedSettings.filter = self.filter;
    if (MountJournal) then
        MountJournalEnhancedSettings.filter.source = MountJournal.filterTypes;
    end
end

function private:OnUpdate()
    if (MountJournal and not self.initiated) then
        self.initiated = true;
        self:init();
    end
end

function private:OnEvent(event, ...)    
    if (event == "COMPANION_LEARNED" or event == "COMPANION_UNLEARNED") then
        if (self.characterMountsCountFrame) then
            self.characterMountsCountFrame.uniqueCount:SetText(GetNumCompanions("MOUNT"));
        end
        return;
    end
    
    if (event == "ACHIEVEMENT_EARNED") then
        if (self.mountAchievementStatusFrame) then
            self.mountAchievementStatusFrame.staticText:SetText(self:getAchievementPoints());
        end
        return;
    end
    
    if (event == "SPELL_UPDATE_USABLE") then
        if (PetJournalParent and PetJournalParent:IsShown()) then
            MountJournal_DirtyList(MountJournal);
            MountJournal_UpdateMountList();
        end
        return;
    end
end

MountJournalEnhanced = {
    Load = function(self, ...) private:Load(...) end,
    Unload = function(self, ...) private:Unload(...) end,
    OnUpdate = function(self, ...) private:OnUpdate(...) end,
    OnEvent = function(self, ...) private:OnEvent(...) end,
};

function MountJournalEnhanced_OnLoad(self)
    MountJournalEnhancedMainFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
    MountJournalEnhancedMainFrame:RegisterEvent("PLAYER_LEAVING_WORLD");
    MountJournalEnhancedMainFrame:RegisterEvent("ADDON_LOADED");
    MountJournalEnhancedMainFrame:RegisterEvent("COMPANION_LEARNED");
    MountJournalEnhancedMainFrame:RegisterEvent("COMPANION_UNLEARNED");
    MountJournalEnhancedMainFrame:RegisterEvent("ACHIEVEMENT_EARNED");
    MountJournalEnhancedMainFrame:RegisterEvent("SPELL_UPDATE_USABLE");
end

function MountJournalEnhanced_OnUpdate(self)
    MountJournalEnhanced:OnUpdate();
end

function MountJournalEnhanced_OnEvent(self, event, ...)
    if (event == "PLAYER_ENTERING_WORLD") then
        MountJournalEnhanced:Load();
        return;
    end

    if (event == "PLAYER_LEAVING_WORLD") then
        MountJournalEnhanced:Unload();
        return;
    end

    MountJournalEnhanced:OnEvent(event, ...);
end